@@ -21,6 +21,8 @@
 #include <memory>
 #include <gtkmm.h>
 
+namespace Templatizer
+{
 class NewTemplateDialog
 {
  public:
@@ -44,5 +46,6 @@ class NewTemplateDialog
     //helpers
     void update_buttons();
 };
+}
 
 #endif
