@@ -17,6 +17,8 @@
 
 #include "templatizer-template.h"
 
+using namespace Templatizer;
+
 TemplatizerTemplate::TemplatizerTemplate(int i, Glib::ustring n, Glib::ustring t)
   :id(i), name(n), text(t)
 {
@@ -33,9 +35,9 @@ TemplatizerTemplate::~TemplatizerTemplate()
         
 bool TemplatizerTemplate::contains(Glib::ustring t)
 {
-  if (name.find(t) != Glib::ustring::npos)
+  if (name.casefold().find(t.casefold()) != Glib::ustring::npos)
     return true;
-  if (text.find(t) != Glib::ustring::npos)
+  if (text.casefold().find(t.casefold()) != Glib::ustring::npos)
     return true;
   return false;
 }
