@@ -22,9 +22,16 @@
 #include "main.h"
 #include "templatizer-window.h"
 #include "new-template-dialog.h"
+#include "delete-template-confirmation-dialog.h"
 #include "templatizer-template.h"
 #include "template-list.h"
 
+//placeholders for future gettextizing
+#define _(X) (X)
+#define N_(X) (X)
+
+using namespace Templatizer;
+
 TemplatizerWindow* TemplatizerWindow::create()
 {
   TemplatizerWindow *window;
@@ -40,7 +47,7 @@ TemplatizerWindow* TemplatizerWindow::create()
 
 TemplatizerWindow::TemplatizerWindow(BaseObjectType* baseObject, 
                              const Glib::RefPtr<Gtk::Builder>& xml)
- : Gtk::Window(baseObject), name_column("Templates", name_renderer)
+ : Gtk::Window(baseObject), name_column(N_("Templates"), name_renderer)
 {
   nag_on_delete = true;
   signal_size_allocate().connect(sigc::mem_fun(*this, &TemplatizerWindow::on_window_allocated));
@@ -98,7 +105,6 @@ TemplatizerWindow::TemplatizerWindow(BaseObjectType* baseObject,
   search_entry->set_text("");
   search_entry->signal_changed().connect(sigc::mem_fun(*this, &TemplatizerWindow::on_search_entry_changed));
   search_entry->signal_focus_in_event().connect(sigc::mem_fun(*this, &TemplatizerWindow::on_entering_search_entry));
-  search_entry->signal_focus_out_event().connect(sigc::mem_fun(*this, &TemplatizerWindow::on_leaving_search_entry));
   search_entry->signal_icon_press().connect(sigc::mem_fun(*this, &TemplatizerWindow::on_search_icon_clicked));
   xml->get_widget("use_button", use_button);
   use_button->signal_clicked().connect(sigc::mem_fun(*this, &TemplatizerWindow::on_use_clicked));
@@ -115,14 +121,15 @@ TemplatizerWindow::TemplatizerWindow(BaseObjectType* baseObject,
   textview->set_wrap_mode(Gtk::WRAP_WORD);
     
   context_actions = Gtk::ActionGroup::create();
-  context_actions->add(Gtk::Action::create("ContextNew", "New Template ..."),
+  context_actions->add(Gtk::Action::create("ContextNew", 
+                                           N_("New Template ...")),
                        sigc::mem_fun(*this, 
                                      &TemplatizerWindow::on_new_clicked));
-  context_actions->add(Gtk::Action::create("ContextDelete", "Delete"),
+  context_actions->add(Gtk::Action::create("ContextDelete", N_("Delete")),
                        sigc::mem_fun(*this, 
                                      &TemplatizerWindow::on_delete_clicked));
   context_actions->add(Gtk::Action::create("ContextUse", 
-                                           "Use in New Email ..."),
+                                           N_("Use in New Email ...")),
                        sigc::mem_fun(*this, 
                                      &TemplatizerWindow::on_use_clicked));
     ui_manager= Gtk::UIManager::create();
@@ -136,10 +143,13 @@ TemplatizerWindow::TemplatizerWindow(BaseObjectType* baseObject,
                                    "  </popup>"
                                    "</ui>");
     treeview_context_menu = dynamic_cast<Gtk::Menu*>(ui_manager->get_widget("/PopupMenu")); 
+    search_entry->property_secondary_icon_pixbuf() = Gtk::IconTheme::get_default()->load_icon("edit-find-symbolic", 16, Gtk::ICON_LOOKUP_GENERIC_FALLBACK);
 }
 
 void TemplatizerWindow::on_treeview_clicked(GdkEventButton* event)
 {
+  // this is how we display the context menu on the treeview when
+  // we right-click on it.
   if (event->type == GDK_BUTTON_PRESS && event->button == 3)
     treeview_context_menu->popup(event->button, event->time);
 }
@@ -180,7 +190,6 @@ void TemplatizerWindow::on_template_selected()
       text_buffer->set_text(t->get_text());
       update_buttons();
     }
-
 }
 
 void TemplatizerWindow::on_template_activated(const Gtk::TreeModel::Path& path,
@@ -204,7 +213,6 @@ void TemplatizerWindow::on_about_clicked()
   dialog->set_version(PACKAGE_VERSION);
   dialog->set_logo(Gdk::Pixbuf::create_from_file(Main::get_data_path() + "/templatizer.png"));
   dialog->show_all();
-  dialog->set_title("About");
   dialog->set_transient_for(*this);
   dialog->run();
   delete dialog;
@@ -218,9 +226,16 @@ void TemplatizerWindow::on_new_clicked()
   int response = d.run();
   if (response == Gtk::RESPONSE_ACCEPT)
     {
+      // stop the active search.
+      on_search_icon_clicked(Gtk::ENTRY_ICON_SECONDARY, NULL);
+
+      // make a new template.
       Glib::ustring name = d.get_template_name();
       TemplatizerTemplate *t = new TemplatizerTemplate(templates->get_free_id(), name, "");
+      // add it to our list, which saves it to the database.
       templates->push_back(t);
+
+      // display and re-sort our list of displayed template names.
       add_template_row(t);
       sorted_templates_list = Gtk::TreeModelSort::create(templates_list);
       sorted_templates_list->set_sort_column(templates_columns.name, Gtk::SORT_ASCENDING);
@@ -246,7 +261,7 @@ void TemplatizerWindow::on_undo_clicked()
 {
   if (template_entry->property_is_focus())
     {
-      ;
+      ;  // FIXME: wouldn't it be nice if the entry had undo support.
     }
   if (textview->property_is_focus())
     text_buffer->get_undo_manager()->undo();
@@ -296,19 +311,12 @@ void TemplatizerWindow::on_deselect_clicked()
 
 int TemplatizerWindow::nag_about_delete_template()
 {
-  Gtk::Dialog* dialog;
-  Glib::RefPtr<Gtk::Builder> xml
-    = Gtk::Builder::create_from_file(Main::get_glade_path() + 
-                                     "/delete-confirmation-dialog.gtk");
-  xml->get_widget("dialog", dialog);
-  Gtk::CheckButton *nag_checkbutton;
-  xml->get_widget("nag_checkbutton", nag_checkbutton);
-  dialog->set_transient_for(*this);
-
-  int response = dialog->run();
-  if (nag_checkbutton->get_active() == true)
+  DeleteTemplateConfirmationDialog d;
+  d.set_parent_window(*this);
+  int response = d.run();
+  if (d.get_nag_state() == true)
     nag_on_delete = false;
-  dialog->hide();
+  d.hide();
   return response;
 }
 
@@ -343,19 +351,25 @@ void TemplatizerWindow::on_delete_clicked()
           sorted_templates_list->set_sort_column(templates_columns.name, Gtk::SORT_ASCENDING);
           treeview->set_model(sorted_templates_list);
           update_buttons();
-
         }
     }
 }
 
 void TemplatizerWindow::on_textview_changed()
 {
+  // when the template text changes, we update our in-memory template object.
+  // we save the template to the database later when the focus leaves the 
+  // template textview.
   TemplatizerTemplate *t = get_selected_template();
   t->set_text(text_buffer->get_text());
 }
 
 void TemplatizerWindow::on_template_entry_changed()
 {
+  // when the template name changes, we also change the name in our displayed 
+  // list of template names.
+  // we save the template to the database later when the focus leaves the 
+  // template entry.
   Glib::RefPtr<Gtk::TreeSelection> selection = treeview->get_selection();
   Gtk::TreeModel::iterator iterrow = selection->get_selected();
 
@@ -367,29 +381,37 @@ void TemplatizerWindow::on_template_entry_changed()
       row[templates_columns.name] = template_entry->get_text();
     }
 }
+
 void TemplatizerWindow::on_use_clicked()
 {
-  Glib::ustring app = Glib::find_program_in_path("gnome-open");
-  Glib::ustring cmd = app + " mailto:?subject=" +
+  Glib::ustring uri = "mailto:?subject=" +
     Glib::uri_escape_string(template_entry->get_text()) +"&body=" + 
     Glib::uri_escape_string(text_buffer->get_text());
-  Glib::spawn_command_line_async(cmd);
-
+  GError *error = NULL;
+  gtk_show_uri(gdk_screen_get_default(), uri.data(),
+               gtk_get_current_event_time(), &error); 
+  if (error)
+    g_free(error);
 }
 
 void TemplatizerWindow::on_search_entry_changed()
 {
   if (search_entry->get_text() == "")
     {
-      //go back to normal
       fill_templates();
       update_buttons();
       return;
     }
   templates_list->clear();
-  for (TemplateList::iterator i = templates->get_templates().begin(); i != templates->get_templates().end();i++)
+  Glib::signal_idle().connect_once(sigc::bind(sigc::mem_fun(*this, &TemplatizerWindow::search_templates_in_the_background), templates, search_entry->get_text()));
+
+}
+
+void TemplatizerWindow::search_templates_in_the_background(TemplateList *t, Glib::ustring match)
+{
+  for (TemplateList::iterator i = t->get_templates().begin(); i != t->get_templates().end();i++)
     {
-      if ((*i)->contains(search_entry->get_text()))
+      if ((*i)->contains(match))
         add_template_row(*i);
     }
   sorted_templates_list = Gtk::TreeModelSort::create(templates_list);
@@ -458,29 +480,28 @@ void TemplatizerWindow::update_buttons()
 
 void TemplatizerWindow::draw_search_indicator()
 {
+  // this is how we put the Search text on the search entry when we are not
+  // searching.
   if (search_entry->get_text() != "" || search_entry->has_focus())
     return;
   Pango::AttrList attrs;
-  Pango::Attribute weight =  
-    Pango::Attribute::create_attr_weight(Pango::WEIGHT_BOLD);
   Pango::Attribute fg =  Pango::Attribute::create_attr_foreground(0.4*65535.0, 
                                                                   0.4*65535.0, 
                                                                   0.4*65535.0);
   Pango::Attribute italic =  
     Pango::Attribute::create_attr_style(Pango::STYLE_ITALIC);
-  attrs.insert(weight);
   attrs.insert(fg);
   attrs.insert(italic);
   Glib::RefPtr<Pango::Layout> layout = search_entry->get_layout();
   layout->set_attributes(attrs);
-  layout->set_text("Search");
+  layout->set_text(N_("Search"));
 }
 
 void TemplatizerWindow::fill_templates()
 {
   templates_list->clear();
-  for (TemplateList::iterator i = templates->get_templates().begin(); i != templates->get_templates().end();
-       i++)
+  for (TemplateList::iterator i = templates->get_templates().begin(); 
+       i != templates->get_templates().end(); i++)
     add_template_row(*i);
 }
 
@@ -492,19 +513,15 @@ void TemplatizerWindow::load_saved_templates()
 
 void TemplatizerWindow::add_template_row(TemplatizerTemplate *t)
 {
+  // put a template name into our displayed list of templates 
   Gtk::TreeIter i = templates_list->append();
   (*i)[templates_columns.name] = t->get_name();
   (*i)[templates_columns.template_data] = t;
 }
 
-bool TemplatizerWindow::on_leaving_search_entry(GdkEventFocus *event)
-{
-  update_buttons();
-  return true;
-}
-
 bool TemplatizerWindow::on_leaving_template_entry(GdkEventFocus *event)
 {
+  // when we're done editing the template name, we flush to disk. 
   TemplatizerTemplate *t = get_selected_template();
   if (t && templates)
     templates->save(t);
@@ -513,6 +530,7 @@ bool TemplatizerWindow::on_leaving_template_entry(GdkEventFocus *event)
 
 bool TemplatizerWindow::on_leaving_textview(GdkEventFocus *event)
 {
+  // when we're done editing the template text, we flush to disk. 
   TemplatizerTemplate *t = get_selected_template();
   if (t && templates)
     templates->save(t);
@@ -521,6 +539,7 @@ bool TemplatizerWindow::on_leaving_textview(GdkEventFocus *event)
   
 bool TemplatizerWindow::on_entering_search_entry(GdkEventFocus *event)
 {
+  // when we click on the search entry, we clear the active template.
   treeview->get_selection()->unselect_all();
   update_buttons();
   return true;
@@ -528,11 +547,14 @@ bool TemplatizerWindow::on_entering_search_entry(GdkEventFocus *event)
   
 void TemplatizerWindow::on_window_allocated(Gtk::Allocation & allocation)
 {
+  // this is a hack to make the "Search" text appear on the search entry.
   update_buttons();
 }
 
 void TemplatizerWindow::cell_data_name(Gtk::CellRenderer *renderer, const Gtk::TreeIter &i)
 {
+  // we specially render the textview column, so that it is ellipsized,
+  // according to the controlling pane.
   Gtk::CellRendererText*cell = dynamic_cast<Gtk::CellRendererText*>(renderer);
   cell->property_ellipsize() = Pango::ELLIPSIZE_END;
   cell->property_text() =  (*i)[templates_columns.name];
