@@ -23,6 +23,8 @@
 #include <gtkmm.h>
 #include <gtksourceviewmm.h>
 
+namespace Templatizer
+{
 class TemplatizerTemplate;
 class TemplateList;
 
@@ -103,7 +105,6 @@ class TemplatizerWindow: public Gtk::Window
     void on_search_icon_clicked(Gtk::EntryIconPosition pos, const GdkEventButton *button);
     void on_treeview_clicked(GdkEventButton* event);
     bool on_leaving_template_entry(GdkEventFocus *event);
-    bool on_leaving_search_entry(GdkEventFocus *event);
     bool on_leaving_textview(GdkEventFocus *event);
     bool on_entering_search_entry(GdkEventFocus *event);
     void on_window_allocated(Gtk::Allocation & allocation);
@@ -118,10 +119,12 @@ class TemplatizerWindow: public Gtk::Window
     void fill_templates();
     void draw_search_indicator();
     int nag_about_delete_template();
+    void search_templates_in_the_background(TemplateList *t, Glib::ustring match);
 
     //DATA
     TemplateList *templates;
     bool nag_on_delete; // e.g. do you want to delete this template?
 };
+}
 
 #endif // TEMPLATIZER_WINDOW_H
